package cz.muni.pa165.generated.device_management;

import cz.muni.pa165.generated.device_management.api.DeviceManagementApiDelegate;
import cz.muni.pa165.generated.device_management.facade.DeviceFacade;
import cz.muni.pa165.generated.device_management.model.DeviceResponse;
import cz.muni.pa165.generated.device_management.model.DeviceResponseList;
import cz.muni.pa165.generated.device_management.model.NewDevice;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.webjars.NotFoundException;


@Component
public class DeviceManagementServer implements DeviceManagementApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(DeviceManagementServer.class);
    private final DeviceFacade deviceFacade;
    private final MeterRegistry meterRegistry;
    private final Counter requestCounter;

    @Autowired
    public DeviceManagementServer(DeviceFacade deviceFacade, MeterRegistry meterRegistry) {
        this.deviceFacade = deviceFacade;
        this.meterRegistry = meterRegistry;
        this.requestCounter = this.meterRegistry.counter("DeviceManagementServer.requestCounter");
    }

    @Override
    public ResponseEntity<DeviceResponse> create(NewDevice newDevice) {
        log.debug("Creating device with name: " + newDevice.getName() + "\n" +
                "MAC address: " + newDevice.getMacAddress() + "\n" +
                "House ID: " + newDevice.getHouseId() + "\n");
        requestCounter.increment();
        try {
            var response = deviceFacade.createDevice(newDevice);
            return new ResponseEntity<>(response, HttpStatus.CREATED);
        } catch (NotFoundException e) {
            log.debug("Device with MAC address: {} already exists.", newDevice.getMacAddress());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<Void> delete(String deviceId) {
        log.debug("Deleting device with ID: " + deviceId + "\n");
        requestCounter.increment();
        try {
            deviceFacade.deleteDevice(deviceId);
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        } catch (NotFoundException e) {
            log.debug("Device with id: {} not found", deviceId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

    }

    @Override
    public ResponseEntity<DeviceResponse> get(String deviceId) {
        log.debug("Getting device with ID: " + deviceId + "\n");
        requestCounter.increment();
        try {
            var response = deviceFacade.getDevice(deviceId);
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (NotFoundException e) {
            log.debug("Device with id: {} not found", deviceId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<DeviceResponse> patch(String deviceId, NewDevice newDevice) {
        log.debug("Updating device with MAC address: " + deviceId + "\n" +
                "to MAC address: " + newDevice.getMacAddress() + "\n" +
                "name: " + newDevice.getName() + "\n");
        requestCounter.increment();
        try {
            var response = deviceFacade.patchDevice(deviceId, newDevice);
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (NotFoundException e) {
            log.debug("No device with MAC address: {} found", deviceId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<DeviceResponse> getLastDevice(String houseId) {
        log.debug("Getting last created device with house ID: " + houseId + "\n");
        requestCounter.increment();
        try {
            var response = deviceFacade.getLastCreatedDeviceByHouseId(houseId);
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (NotFoundException e) {
            log.debug("No devices for house: {} found", houseId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<DeviceResponseList> getDevices(String houseId) {
        log.debug("Getting device with house ID: " + houseId + "\n");
        requestCounter.increment();
        try {
            var response = deviceFacade.getDevicesByHouseId(houseId);
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (NotFoundException e) {
            log.debug("No devices for house: {} found", houseId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
}
