package cz.muni.pa165.generated.device_management;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.pa165.generated.device_management.facade.DeviceFacadeImpl;
import cz.muni.pa165.generated.device_management.model.NewDevice;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.web.servlet.MockMvc;

import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@ExtendWith(MockitoExtension.class)
@Transactional
@Rollback
class DeviceManagementServerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @Test
    void createDevice() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);

        var response = mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString();

        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"macAddress\":\"macAddress\""));
        assertTrue(response.contains("\"houseId\":\"someHouseID\""));
    }

    @Test
    void createDeviceMacAddressAlreadyExist() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);

        mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated());

        mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isNotFound());
    }

    @Test
    void getNonExistingDevice() throws Exception {
        mockMvc.perform(get("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceId", "nonExistingId"))
                .andExpect(status().isNotFound());
    }

    @Test
    void getDevice() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);
        var deviceId = mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        mockMvc.perform(get("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceId", deviceId))
                .andExpect(status().isOk());
    }

    @Test
    void patchDevice() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);
        var deviceId = mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        var updateDevice = new NewDevice();
        updateDevice.setMacAddress("macAddress");
        updateDevice.setName("updateName");
        updateDevice.setHouseId("houseID");

        String jsonUpdateDevice = objectMapper.writeValueAsString(updateDevice);

        String response = mockMvc.perform(put("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceId", deviceId)
                        .content(jsonUpdateDevice))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertTrue(response.contains("\"name\":\"updateName\""));
        assertTrue(response.contains("\"macAddress\":\"macAddress\""));
        assertTrue(response.contains("\"houseId\":\"houseID\""));
    }

    @Test
    void patchDeviceNonExisting() throws Exception {
        mockMvc.perform(put("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceMacAddress", "nonExistingDeviceId")
                        .content(""))
                .andExpect(status().isBadRequest());
    }

    @Test
    void deleteDevice() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);
        var deviceId = mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        mockMvc.perform(delete("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceId", deviceId))
                .andExpect(status().isNoContent());
    }

    @Test
    void deleteDeviceNonExisting() throws Exception {
        mockMvc.perform(delete("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("deviceId", "1"))
                .andExpect(status().isNotFound());
    }

    @Test
    void getDevicesForExistingHouse() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);
        mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated());

        var response = mockMvc.perform(get("/api/devices")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("houseId", "someHouseID"))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"macAddress\":\"macAddress\""));
        assertTrue(response.contains("\"houseId\":\"someHouseID\""));
    }

    @Test
    void getDevicesForNonExistingHouse() throws Exception {
        mockMvc.perform(get("/api/devices")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("houseId", "nonExistingHouseID"))
                .andExpect(status().isNotFound());
    }

    @Test
    void getLastDevice() throws Exception {
        var newDevice = new NewDevice();
        newDevice.setMacAddress("macAddress");
        newDevice.setName("name");
        newDevice.setHouseId("someHouseID");

        String jsonNewDevice = objectMapper.writeValueAsString(newDevice);
        mockMvc.perform(post("/api/device")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewDevice))
                .andExpect(status().isCreated());

        var response = mockMvc.perform(get("/api/lastDevice")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("houseId", "someHouseID"))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"macAddress\":\"macAddress\""));
        assertTrue(response.contains("\"houseId\":\"someHouseID\""));
    }

    @Test
    void getLastDeviceNonExistingHouse() throws Exception {
        mockMvc.perform(get("/api/lastDevice")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("houseId", "nonExistingHouseID"))
                .andExpect(status().isNotFound());
    }
}