package cz.muni.pa165.generated.power_consumption;

import cz.muni.pa165.generated.power_consumption.Service.PowerConsumptionServiceImpl;
import cz.muni.pa165.generated.power_consumption.api.PowerConsumptionApiDelegate;
import cz.muni.pa165.generated.power_consumption.data.model.DeviceConsumption;
import cz.muni.pa165.generated.power_consumption.model.DeviceConsumptionRequest;
import cz.muni.pa165.generated.power_consumption.model.DeviceConsumptionResponse;
import cz.muni.pa165.generated.power_consumption.model.DeviceConsumptionSample;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import org.hibernate.ObjectNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.time.OffsetDateTime;
import java.util.List;

@Component
public class PowerConsumptionServer implements PowerConsumptionApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(PowerConsumptionServer.class);
    private final MeterRegistry meterRegistry;
    private final Counter requestCounter;
    private final PowerConsumptionServiceImpl powerConsumptionService;

    @Autowired
    public PowerConsumptionServer(PowerConsumptionServiceImpl powerConsumptionService, MeterRegistry meterRegistry) {
        this.powerConsumptionService = powerConsumptionService;
        this.meterRegistry = meterRegistry;
        this.requestCounter = meterRegistry.counter("PowerConsumptionServer.requestCounter");
    }

    @Override
    public ResponseEntity<DeviceConsumptionResponse> createPowerConsumption(DeviceConsumptionRequest newDeviceConsumption) {
        log.debug("Creating new power consumption.");
        requestCounter.increment();
        var newConsumption = new DeviceConsumption();
        newConsumption.setDeviceMacAddress(newDeviceConsumption.getDeviceMacAddress());
        newConsumption.setConsumption(newDeviceConsumption.getConsumption());
        newConsumption.setUnit(newDeviceConsumption.getUnit());
        newConsumption.setDateTime(newDeviceConsumption.getDateTime());
        var result = powerConsumptionService.createDeviceConsumption(newConsumption);

        var response = new DeviceConsumptionResponse()
                .id(result.getId())
                .deviceMacAddress(result.getDeviceMacAddress())
                .consumption(result.getConsumption())
                .unit(result.getUnit())
                .dateTime(result.getDateTime());
        return new ResponseEntity<>(response, HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<Void> deletePowerConsumption(String id) {
        try {
            log.debug("Deleted power consumption: {}", id);
            requestCounter.increment();
            this.powerConsumptionService.deleteDeviceConsumption(id);
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        } catch (ObjectNotFoundException e) {
            log.debug("Power consumption with id: {} not found.", id);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<DeviceConsumptionResponse> getPowerConsumption(String id) {
        try {
            log.debug("Fetching power consumption: {}", id);
            requestCounter.increment();
            var deviceConsumption = this.powerConsumptionService.getConsumption(id);

            var response = new DeviceConsumptionResponse()
                    .id(deviceConsumption.getId())
                    .deviceMacAddress(deviceConsumption.getDeviceMacAddress())
                    .consumption(deviceConsumption.getConsumption())
                    .unit(deviceConsumption.getUnit())
                    .dateTime(deviceConsumption.getDateTime());
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (ObjectNotFoundException e) {
            log.debug("Power consumption with id: {} not found.", id);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<List<DeviceConsumptionResponse>> getPowerConsumptionInTimeSpan(String deviceMacAddress,
                                                                                         OffsetDateTime startTime,
                                                                                         OffsetDateTime endTime) {
        log.debug("Fetching power consumption with device id: {} in time span {} - {} ", deviceMacAddress, startTime, endTime);
        requestCounter.increment();

        var result = this.powerConsumptionService.getPowerConsumption(deviceMacAddress, startTime, endTime);
        List<DeviceConsumptionResponse> response = result.stream().map(
                c -> new DeviceConsumptionResponse()
                        .id(c.getId())
                        .deviceMacAddress(c.getDeviceMacAddress())
                        .consumption(c.getConsumption())
                        .unit(c.getUnit())
                        .dateTime(c.getDateTime())
        ).toList();
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<DeviceConsumptionResponse>> listAllConsumptionsWithSpecificDevice(String deviceId) {
        log.debug("Fetching power consumption with device id: {}", deviceId);
        requestCounter.increment();

        var result = this.powerConsumptionService.listAllConsumptionsWithSpecificDevice(deviceId);
        List<DeviceConsumptionResponse> response = result.stream().map(
                c -> new DeviceConsumptionResponse()
                        .id(c.getId())
                        .deviceMacAddress(c.getDeviceMacAddress())
                        .consumption(c.getConsumption())
                        .unit(c.getUnit())
                        .dateTime(c.getDateTime())
        ).toList();
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<String>> listAllDevices() {
        log.debug("Fetching all device id with some consumption");
        requestCounter.increment();
        return new ResponseEntity<>(this.powerConsumptionService.listAllDevices(), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<DeviceConsumptionResponse>> listAllPowerConsumptions() {
        log.debug("Fetching all power consumptions");
        requestCounter.increment();
        var result = this.powerConsumptionService.listAllConsumptions();
        List<DeviceConsumptionResponse> response = result.stream().map(
                c -> new DeviceConsumptionResponse()
                        .id(c.getId())
                        .deviceMacAddress(c.getDeviceMacAddress())
                        .consumption(c.getConsumption())
                        .unit(c.getUnit())
                        .dateTime(c.getDateTime())
        ).toList();
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> updatePowerConsumption(String id, DeviceConsumptionRequest deviceConsumption) {
        try {
            log.debug("Updating power consumption: {}", id);
            requestCounter.increment();
            var updateConsumption = new DeviceConsumption();
            updateConsumption.setDeviceMacAddress(deviceConsumption.getDeviceMacAddress());
            updateConsumption.setConsumption(deviceConsumption.getConsumption());
            updateConsumption.setUnit(deviceConsumption.getUnit());
            updateConsumption.setDateTime(deviceConsumption.getDateTime());

            this.powerConsumptionService.updateDeviceConsumption(id, updateConsumption);

            return new ResponseEntity<>(HttpStatus.OK);
        } catch (ObjectNotFoundException e) {
            log.debug("Power consumption with id: {} not found.", id);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<Void> postDeviceConsumptionData(
            DeviceConsumptionSample deviceConsumptionSample) {
        log.debug("Posting consumption sample ");
        requestCounter.increment();
        var newConsumption = new DeviceConsumption();
        newConsumption.setDeviceMacAddress(deviceConsumptionSample.getDeviceMacAddress());
        newConsumption.setConsumption(deviceConsumptionSample.getConsumption());
        newConsumption.setUnit(deviceConsumptionSample.getUnit());

        this.powerConsumptionService.createDeviceConsumption(newConsumption);
        return new ResponseEntity<>(HttpStatus.CREATED);
    }
}
