package cz.muni.pa165.generated.power_control;

import cz.muni.pa165.generated.power_control.api.PowerControlApiDelegate;
import cz.muni.pa165.generated.power_control.facade.HouseFacade;
import cz.muni.pa165.generated.power_control.model.HouseResponse;
import cz.muni.pa165.generated.power_control.model.HouseResponseList;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import jakarta.persistence.EntityNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.webjars.NotFoundException;

import java.util.NoSuchElementException;

@Component
public class PowerControlServer implements PowerControlApiDelegate {
    private final HouseFacade houseFacade;
    private static final Logger log = LoggerFactory.getLogger(PowerControlServer.class);
    private final MeterRegistry meterRegistry;
    private final Counter requestCounter;

    @Autowired
    public PowerControlServer(HouseFacade houseFacade, MeterRegistry meterRegistry) {
        this.houseFacade = houseFacade;
        this.meterRegistry = meterRegistry;
        this.requestCounter = this.meterRegistry.counter("PowerControlServer.requestCounter");
    }

    @Override
    public ResponseEntity<HouseResponse> switchPowerInHouse(String id) {
        try {
            requestCounter.increment();
            HouseResponse response = houseFacade.switchPower(id);
            log.debug("Power of house {} switched {}", id, response.getEnabled());
            return new ResponseEntity<>(response, HttpStatus.OK);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("House {} not found", id);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<HouseResponseList> getHousesByOwner(String ownerId) {
        try{
            log.debug("Fetching houses of owner: {}", ownerId);
            requestCounter.increment();
            return new ResponseEntity<>(this.houseFacade.getAllByOwnerId(ownerId), HttpStatus.OK);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("House with id: {} not found", ownerId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<HouseResponse> create(NewHouse newHouse) {
        try {
            requestCounter.increment();
            var houseResponse = this.houseFacade.create(newHouse);
            log.debug("Creating house: {}", newHouse);
            return new ResponseEntity<>(houseResponse, HttpStatus.CREATED);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("User not found: {}", e.getMessage());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<HouseResponse> get(String houseId) {
        try{
            log.debug("Fetching house: {}", houseId);
            requestCounter.increment();
            return new ResponseEntity<>(this.houseFacade.get(houseId), HttpStatus.OK);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("House with id: {} not found", houseId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<HouseResponse> update(String id,UpdateHouse updateHouse) {
        try {
            requestCounter.increment();
            var houseResponse = this.houseFacade.update(id,updateHouse);
            log.debug("Updating house: {} with data: {}", id, updateHouse);
            return new ResponseEntity<>(houseResponse, HttpStatus.OK);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("User not found: {}", e.getMessage());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<Void> delete(String houseId) {
        try{
            requestCounter.increment();
            this.houseFacade.delete(houseId);
            log.debug("Deleted house: {}", houseId);
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        } catch (NoSuchElementException | EntityNotFoundException | NotFoundException e) {
            log.debug("House with id: {} not found", houseId);
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
}
